package services

import "eta/eta_mini_crm/models"

// 获得全部的权限结构列表
func GetChartPermissionList() (publicMap, privateMap map[string]*models.ChartPermissionList, err error) {
	items, err := models.GetChildChartPermissionListById(0)
	if err != nil {
		return
	}
	publicMap = make(map[string]*models.ChartPermissionList)
	privateMap = make(map[string]*models.ChartPermissionList)
	for _, cpl := range items {
		childPermissionList, e := models.GetChildChartPermissionListById(cpl.ChartPermissionId)
		if e != nil {
			return nil, nil, e
		}
		publicTmp := copyChartPermissionList(cpl)
		privateTmp := copyChartPermissionList(cpl)

		for _, childCpl := range childPermissionList {
			if childCpl.IsPublic == 1 {
				// 公有权限
				publicTmp.Child = append(publicTmp.Child, childCpl)
			} else {
				// 私有权限
				privateTmp.Child = append(privateTmp.Child, childCpl)
			}
		}
		if len(publicTmp.Child) > 0 {
			publicMap[cpl.ChartPermissionName] = publicTmp
		}
		if len(privateTmp.Child) > 0 {
			privateMap[cpl.ChartPermissionName] = privateTmp
		}
	}
	return
}

func copyChartPermissionList(src *models.ChartPermissionList) *models.ChartPermissionList {
	tmp := &models.ChartPermissionList{
		ChartPermissionId:     src.ChartPermissionId,
		ChartPermissionName:   src.ChartPermissionName,
		PermissionName:        src.PermissionName,
		Sort:                  src.Sort,
		Enabled:               src.Enabled,
		CreatedTime:           src.CreatedTime,
		LastUpdatedTime:       src.LastUpdatedTime,
		TeleconferenceSort:    src.TeleconferenceSort,
		Remark:                src.Remark,
		ClassifyName:          src.ClassifyName,
		ProductName:           src.ProductName,
		ImageURL:              src.ImageURL,
		ShowType:              src.ShowType,
		IsOther:               src.IsOther,
		IsReport:              src.IsReport,
		CygxAuth:              src.CygxAuth,
		PermissionType:        src.PermissionType,
		YbImgUrl:              src.YbImgUrl,
		ProductPermissionName: src.ProductPermissionName,
		PriceDrivenState:      src.PriceDrivenState,
		ImageUrlM:             src.ImageUrlM,
		ParentId:              src.ParentId,
		IsPublic:              src.IsPublic,
		Child:                 src.Child,
	}
	return tmp
}
