package services

import (
	"eta/eta_mini_crm/models"
	"strconv"
	"strings"
)

func GetSysDepartmentPathById(sysDepartmentId int) (path string, err error) {
	sysDepartment, err := models.GetSysDepartmentById(sysDepartmentId)
	if err != nil {
		return
	}

	path = sysDepartment.SysDepartmentName
	curSysDepartmentId := sysDepartment.ParentId
	for curLevel := sysDepartment.Level - 1; curLevel >= 1; curLevel-- {
		nextSysDepartment, er := models.GetSysDepartmentListByParentId(curSysDepartmentId)
		if er != nil {
			return
		}
		curSysDepartmentId = nextSysDepartment.ParentId
		path = nextSysDepartment.SysDepartmentName + "/" + path
	}
	return
}

func GetSysDepartmentList() ([]*models.SysDepartmentList, error) {
	departmentList, err := models.GetSysDepartmentListByLevel(1)
	if err != nil {
		return nil, err
	}
	for _, dep := range departmentList {
		childDepartmentList, err := models.GetChildSysDepartmentListById(dep.SysDepartmentId)
		if err != nil {
			return nil, err
		}
		dep.Child = childDepartmentList
		for _, cdep := range childDepartmentList {
			subDepartmentList, err := models.GetChildSysDepartmentListById(cdep.SysDepartmentId)
			if err != nil {
				return nil, err
			}
			cdep.Child = subDepartmentList
		}
	}
	return departmentList, nil
}

func DeleteSysDepartmentById(sysDepartmentId, level int) (err error) {
	sysDepartmentIds := make([]string, 0)
	sysDepartmentIds = append(sysDepartmentIds, strconv.Itoa(sysDepartmentId))
	var curIds = sysDepartmentIds
	var curStringIds string
	for i := level; i < 3; i++ {
		curStringIds = strings.Join(curIds, ",")
		sysDepartments, er := models.GetChildSysDepartmentListByIds(curStringIds)

		if er != nil {
			return er
		}
		curIds = make([]string, 0)
		for _, dep := range sysDepartments {
			sysDepartmentIds = append(sysDepartmentIds, strconv.Itoa(dep.SysDepartmentId))
			curIds = append(curIds, strconv.Itoa(dep.SysDepartmentId))
		}
	}
	err = models.DeleteSysDepartmentByIdV2(sysDepartmentIds, level)
	return
}

func GetSysDepartmentPathIdsById(sysDepartmentId int) (pathIds []int, err error) {
	sysDepartment, err := models.GetSysDepartmentById(sysDepartmentId)
	if err != nil {
		return
	}

	pathIds = make([]int, 3)
	pathIds[sysDepartment.Level-1] = sysDepartmentId
	curSysDepartmentId := sysDepartment.ParentId
	for curLevel := sysDepartment.Level - 1; curLevel >= 1; curLevel-- {
		upSysDepartment, er := models.GetSysDepartmentListByParentId(curSysDepartmentId)
		if er != nil {
			return
		}
		curSysDepartmentId = upSysDepartment.ParentId
		pathIds[curLevel-1] = upSysDepartment.SysDepartmentId
	}
	return
}
